# Copyright 2021 (c) Anna Schumaker.
import audio
import lib
import sidebar
import unittest
from gi.repository import Gtk
from . import keyboard
from . import pane
from . import window

class TestWindow(unittest.TestCase):
    def setUp(self):
        lib.settings.reset()

    def test_init(self):
        win = window.Window()
        self.assertIsInstance(win, Gtk.ApplicationWindow)
        self.assertIsInstance(win.get_titlebar(), audio.Header)
        self.assertIsInstance(win.get_child(), pane.Pane)

        self.assertEqual(win.get_icon_name(), "emmental")
        self.assertEqual(win.get_title(), lib.version.string())
        self.assertEqual(keyboard.Event.get_widget(), win)
        win.remove_controller(keyboard.Event)

    def test_size(self):
        win = window.Window()
        self.assertEqual(lib.settings.get_int("window.width"), 1400)
        self.assertEqual(lib.settings.get_int("window.height"), 800)
        self.assertEqual(win.get_default_size(), (1400, 800))

        win.set_default_size(1000, 500)
        self.assertEqual(lib.settings.get_int("window.width"), 1000)
        self.assertEqual(lib.settings.get_int("window.height"), 500)
        win.remove_controller(keyboard.Event)

    def test_maximize(self):
        win = window.Window()
        self.assertFalse(lib.settings.get_bool("window.maximized"))
        (width, height) = win.get_default_size()

        win.maximize()
        self.assertTrue(lib.settings.get_bool("window.maximized"))
        self.assertEqual(lib.settings.get_int("window.width"), width)
        self.assertEqual(lib.settings.get_int("window.height"), height)

        win.unmaximize()
        self.assertEqual(lib.settings.get_int("window.width"), width)
        self.assertEqual(lib.settings.get_int("window.height"), height)
        self.assertFalse(lib.settings.get_bool("window.maximized"))
        win.remove_controller(keyboard.Event)
